#!/usr/bin/perl
#
# (c) astx (at) aws-it.at, 2019
#
# $Id: calculate_soa_data_from_log_log_plot.pl 1557 2019-01-03 11:48:03Z sta $

use strict;

use Getopt::Long;

our $soa_data=undef;

my $rev='$Rev: 1557 $';
$rev=~s/[^0-9]//g;
my $version="0.1.$rev";
#my $db_file='calculate_soa_data_from_log_log_plot.soa_db';
#my $input_file='calculate_soa_data_from_log_log_plot.input';
my $volt=70;

GetOptions(
#	"database=s"	=>	\$db_file,
	"examples"	=>	\&show_examples,
#	"input=s"	=>	\$input_file,
	"help|?"	=>	\&show_help,
	"version|v"	=>	\&show_version,
);

&load_soa_db();

## load soa db file
#if (-f "./$db_file") {
#	require "./$db_file";
#} else {
#	die "soa database file <$db_file> not found";
#}

## load input file
#if (-f "./$input_file") {
#	require "./$input_file";
#} else {
#	die "your soa input/config file <$input_file> wasn't found";
#}

#---------------------------------------------------------------------------
sub get_soa_data_from_points
#---------------------------------------------------------------------------
{
	# we need 2 points on log log plot and calculate the value of y where x = 1 which is called k
	# delta log y = log y2 - log y1
	# delta log x = log x2 - log x1
	# m = delta logy / delta logx
	my $ref=shift;

	my $x1=shift;
	my $y1=shift;
	my $x2=shift;
	my $y2=shift;

	my $x_from=shift;
	my $x_to=shift;

	#calculate slope
	my $d_logx=log($x2) - log($x1);
	my $d_logy=log($y2) - log($y1);
	my $m=$d_logy / $d_logx;

	# print "slope m = $m\n";

	# calculate intercept on y axis to get constant k
	my $ln_a=log($y2) - $m*log($x2);
	# calculate anti log
	my $k=exp($ln_a);
	# print "log(k):  $ln_a => antilog(k): $k\n";

	# now we have k and m and can calculate y values by iterating x:
	# y = k * x**m
	for my $x ($x_from .. $x_to) {
		my $y=$k * $x ** $m;
		my $oy=sprintf ("%0.2f", $y);
		# print STDERR "$x\t$oy\n";
		$ref->{$x}=$oy;
	}
}

&print_compose_data(uc($ARGV[0]));

# -----------------------------------------------------------------------------
sub print_compose_data
# -----------------------------------------------------------------------------
{
	my $dev=shift;

	if (defined $soa_data->{$dev}) {
		my $e=$soa_data->{$dev};
		foreach my $t (@{$e}) {
			my $soa={};
			my $tc=$t->{'tc'};
			print STDERR "$dev: found Tc entry for $tc\n";
			foreach my $l (@{$t->{'data'}}) {
				print STDERR "L: @{$l}\n";
				&get_soa_data_from_points($soa, @{$l});
			}
			my $x;
			my @x;
			my @y;
			for($x=1; $x < 201; $x++) {
				if (defined $soa->{$x}) {
					# print STDERR "$x\t$soa->{$x}\n";
					push(@x, $x);
					push(@y, $soa->{$x});
				}
			}
			my $ldev=lc($dev);
			print "compose $ldev\_soa$tc\_x values @x\n";
			print "compose $ldev\_soa$tc\_y values @y\n";
		}
	} else {
		print STDERR "no such device '$dev' in database\n";
	}
}

# -----------------------------------------------------------------------------
sub show_help
# -----------------------------------------------------------------------------
{
	print STDERR qq~Usage: $0 [OPTION]... [FILE]...
Generate 'compose' data lines for usage in ngspice simulation software

	--help			this help
	--examples		show examples
	--version		show version

	calculate_soa_data_from_log_log_plot V$version
	(c) 2016 - 2018, astx\@aws-it.at
~;
	exit;
}

# -----------------------------------------------------------------------------
sub show_version
# -----------------------------------------------------------------------------
{
	print "$version\n";
	exit;
}

# -----------------------------------------------------------------------------
sub show_examples
# -----------------------------------------------------------------------------
{

print STDERR qq~
# -----------------------------------------------------------------------------
# EXAMPLE SOA DATA OUTPUT
# -----------------------------------------------------------------------------
	#
	# PGMR: (c) 2016 - 2018, astx (at) aws-it.at
	#

	# to make perl happy you need this here:
	1;
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# GENERATE SOA DATA FOR ngspice scripting
# -----------------------------------------------------------------------------

./calculate_soa_data_from_log_log_plot.pl IXTH80N20L

~;
}

# -----------------------------------------------------------------------------
sub load_soa_db
# -----------------------------------------------------------------------------
{
	
	our $soa_data	=	{
		'IXTH80N20L'	=>	[
			{	'tc'	=>	'25',
				'data'	=>	[
							# IXTH80N20L has 3 lines for tc25
							[ 1, 20, 10, 100, 1, 7 ],
							[ 7, 77.93, 500, 1, 7, 80 ],
							[ 80, 6.49, 800, 0.1, 80, 200 ],
				],
				'desc'	=>	'DC operation single pulse Tc 25',
			},
			{
				'tc'	=>	'75',
				'data'	=>	[
							# IXTH80N20L has 3 lines for tc75
							[ 1, 20, 10, 100, 1, 5 ],
							[ 5, 63, 80, 3.9, 5, 80 ],
							[ 80, 3.9, 200, 0.74, 80, 200 ],
				],
				'desc'	=>	'DC operation single pulse Tc 75',
			},
		],
		'IXTH48P20P'	=>	[
			{	'tc'	=>	'25',
				'data'	=>	[
							# IXTH48P20P has 3 lines for tc25
							[ 1, 7.4, 30, 90, 1, 11 ],
							[ 5, 90, 490, 1, 11, 160 ],
							[ 160, 3, 200, 2, 160, 200 ],
				],
				'desc'	=>	'DC operation single pulse Tc 25',
			},
			{
				'tc'	=>	'70',
				'data'	=>	[
							# IXTH48P20P has 3 lines for tc75
							[ 1, 7.4, 30, 90, 1, 8 ],
							[ 3, 100, 300, 1, 8, 160 ],
							[ 10, 90, 240, 1, 160, 200 ],
				],
				'desc'	=>	'DC operation single pulse Tc 75',
			},
		],
	};

	# aliases
#	$soa_data->{'2SC4793_A'}=$soa_data->{'2SC4793'};
#	$soa_data->{'2SA1837_A'}=$soa_data->{'2SA1837'};

#	$soa_data->{'2SC5200'}=$soa_data->{'TTC5200'};
#	$soa_data->{'2SC5200_k'}=$soa_data->{'TTC5200'};

#	$soa_data->{'2SA1943'}=$soa_data->{'TTA1943'};
#	$soa_data->{'2SA1943_k'}=$soa_data->{'TTA1943'};

}
